/* See {allgraphs.h} */
/* Last edited on 2007-01-04 03:49:01 by stolfi */

#define _GNU_SOURCE

#include <allgraphs.h>

#include <fltgraph.h>
#include <iagraph.h>
#include <aagraph.h>
#include <flt.h>
#include <ia.h>
#include <aa.h>
#include <ps.h>
#include <bool.h>
#include <jsstring.h>
#include <jsfile.h>

#include <math.h>
#include <stdio.h>
#include <malloc.h>

/*** INTERNAL PROTOTYPES ***/
  
char *allgraphs_format_parms(
    Float xmin, Float xmax,
    Float ymin, Float ymax,
    int nsub
  );
  
FILE *allgraphs_open_plot
  ( FILE *psfile,
    bool_t epsformat,
    char *fileprefix,
    char *tag,
    char *pagenum,
    char *method,
    char *function,
    char *parmstr,
    double xmin, double xmax,
    double ymin, double ymax,
    int nsub
  );

/* IMPLEMENTATIONS */

char *allgraphs_format_parms(
    Float xmin, Float xmax,
    Float ymin, Float ymax,
    int nsub
  )
  { char *s = NULL;
    asprintf(&s, 
      "x in [%f _ %f]  y in [%f _ %f]\n%d intervals",
      xmin, xmax, ymin, ymax, nsub
    );
    return(s);
  }

void allgraphs_plot(
    char *fileprefix,
    int epsformat,
    char *function,
    eval_fp_t eval_fp,
    eval_ia_t eval_ia,
    diff_ia_t diff_ia,
    eval_aa_t eval_aa,
    Float xmin, Float xmax,
    Float ymin, Float ymax,
    int nsub,
    int nsteps
  )
  { char *parmstr = allgraphs_format_parms(xmin, xmax, ymin, ymax, nsub);
    FILE *psfile = NULL;
    
    Interval xd = (Interval){xmin, xmax};
    Interval yd = (Interval){ymin, ymax};

    if (! epsformat)
      { /* Open single document file: */
        psfile = open_write(txtcat(fileprefix, ".ps"), TRUE);
        ps_begin_document(psfile, "letter");
      }
    
    /*** IA BOX ENCLOSURES ***/
    
    psfile = allgraphs_open_plot
      ( psfile, epsformat, fileprefix, 
        "ia", "ia", "Box enclosures (Interval Arith.)",
        function,  parmstr,
        xmin, xmax,  ymin, ymax,
        nsub
      );
    
    ps_set_pen(psfile, 0.0,0.0,0.0, 0.10, 0.0, 0.0);
    iagraph_plot_boxes(psfile, eval_ia, xd, yd, nsub);
    
    ps_set_pen(psfile, 0.5,0.0,0.0, 0.10, 0.0, 0.0);
    fltgraph_plot(psfile, eval_fp, xd, yd, nsteps);
    
    ps_set_pen(psfile, 0.0,0.0,0.0, 0.10, 0.0, 0.0);
    fltgraph_draw_axes(psfile, xd, yd);
    
    ps_set_pen(psfile, 0.0,0.0,0.0, 0.25, 0.0, 0.0);
    ps_draw_frame(psfile);

    if (epsformat)
      { ps_end_figure(psfile); }
    else
      { ps_end_page(psfile); }
      
    /*** AA BOX ENCLOSURES ***/

    psfile = allgraphs_open_plot
      ( psfile, epsformat, fileprefix, 
        "ar", "ar", "Box enclosures (Affine Arith.)",
        function,  parmstr,
        xmin, xmax,  ymin, ymax,
        nsub
      );

    aagraph_plot_boxes(psfile, eval_aa, xd, yd, nsub);
    fltgraph_draw_axes(psfile, xd, yd);
    fltgraph_plot(psfile, eval_fp, xd, yd, nsteps);
    ps_draw_frame(psfile);
    
    if (epsformat) 
      { ps_end_figure(psfile); }
    else
      { ps_end_page(psfile); }
      
    /*** IA INTERVAL-SLOPE ENCLOSURES ***/
    
    if (diff_ia != NULL)
      { psfile = allgraphs_open_plot
          ( psfile, epsformat, fileprefix, 
            "id", "ad", "Interval-slope enclosures (Interval Arith.)",
            function,  parmstr,
            xmin, xmax,  ymin, ymax,
            nsub
          );

        ps_set_pen(psfile, 0.0,0.0,0.0, 0.10, 0.0, 0.0);
        iagraph_plot_butterflies(psfile, eval_ia, diff_ia, xd, yd, nsub);

        ps_set_pen(psfile, 0.5,0.0,0.0, 0.10, 0.0, 0.0);
        fltgraph_plot(psfile, eval_fp, xd, yd, nsteps);

        ps_set_pen(psfile, 0.0,0.0,0.0, 0.10, 0.0, 0.0);
        fltgraph_draw_axes(psfile, xd, yd);

        ps_set_pen(psfile, 0.0,0.0,0.0, 0.25, 0.0, 0.0);
        ps_draw_frame(psfile);

        if (epsformat)
          { ps_end_figure(psfile); }
        else
          { ps_end_page(psfile); }
      }

    /*** AA BAND ENCLOSURES ***/

    psfile = allgraphs_open_plot
      ( psfile, epsformat, fileprefix, 
        "aa", "aa", "Band enclosures (Affine Arith.)",
        function,  parmstr,
        xmin, xmax,  ymin, ymax,
        nsub
      );
    
    aagraph_plot_paralelograms(psfile, eval_aa, xd, yd, nsub);
    fltgraph_draw_axes(psfile, xd, yd);
    fltgraph_plot(psfile, eval_fp, xd, yd, nsteps);
    ps_draw_frame(psfile);
    
    if (epsformat) 
      { ps_end_figure(psfile); }
    else
      { ps_end_page(psfile); }

    /* FINISH */
    
    if (! epsformat)
      { ps_end_document(psfile); fclose(psfile); }
  }

FILE *allgraphs_open_plot
  ( FILE *psfile,
    bool_t epsformat,
    char *fileprefix,
    char *tag,
    char *pagenum,
    char *method,
    char *function,
    char *parmstr,
    double xmin, double xmax,
    double ymin, double ymax,
    int nsub
  )
  {
    double wx = xmax - xmin;
    double wy = ymax - ymin;
    double wm = (wx > wy ? wx : wy);
    
    double scale = 6.0 * 72.0 / wm;
    double hctr = 4.25 * 72.0;
    double hmin = hctr - scale * wx / 2.0;
    double hmax = hctr + scale * wx / 2.0;
    double vtop = 9.5 * 72.0;
    double vmin = vtop - scale * wy;
    double vmax = vtop;

    if (epsformat)
      { char *filename = NULL;
        asprintf(&filename, "%s-%s.eps", fileprefix, tag); 
        psfile = open_write(filename, TRUE);
        ps_begin_figure(psfile, 0.0, hmax-hmin, 0.0, vmax-vmin);
        ps_set_window(psfile, 
	  xmin, xmax,      ymin, ymax,
          0.0,  hmax-hmin, 0.0,  vmax-vmin,
	  nsub, 1
	);
        free(filename);
      }
    else
      { ps_begin_page(psfile, pagenum);
	ps_set_window(psfile,
	  xmin, xmax, ymin, ymax,
	  hmin,  hmax,  vmin,  vmax,
	  nsub, 1
	);
        ps_add_caption(psfile, "");
        ps_add_caption(psfile, function);
        ps_add_caption(psfile, "");
        ps_add_caption(psfile, method);
        ps_add_caption(psfile, "");
        ps_add_caption(psfile, parmstr);
      }

    return psfile;
  }
