/* See hr2.h */
/* Last edited on 2005-07-10 21:41:57 by stolfi */ 

#include <hr2.h>

#include <r2.h>
#include <r3.h>
#include <r3x3.h>
#include <affirm.h>

#include <math.h>

/* Based on HR2.m3, created 1994-05-04 by J. Stolfi. */

hr2_point_t hr2_from_r2(r2_t *c)
  {
    return (hr2_point_t){{{1.0, c->c[0], c->c[1]}}};
  }

r2_t r2_from_hr2(hr2_point_t *p)
  {
    double w = p->c.c[0];
    demand(w != 0.0, "point at infinity");
    return (r2_t){{p->c.c[1]/w, p->c.c[2]/w}};
  }

sign_t hr2_side(hr2_point_t *p, hr2_line_t *L)
  {
    double dd = r3_dot(&(p->c), &(L->f));
    if (dd > 0.0)
      { return +1; }
    else if (dd < 0.0)
      { return -1; }
    else 
      { return 0; }
  }

sign_t hr2_orient(hr2_point_t *p, hr2_point_t *q, hr2_point_t *r)
  {
    double dd = r3_det(&(p->c), &(q->c), &(r->c));
    if (dd > 0.0)
      { return 1; }
    else if (dd < 0.0)
      { return -1; }
    else 
      { return 0; }
  }

hr2_line_t hr2_join(hr2_point_t *p, hr2_point_t *q)
  {
    r3_t f;
    r3_cross(&(p->c), &(q->c), &f);
    return (hr2_line_t){f};
  }

hr2_point_t hr2_meet(hr2_line_t *K, hr2_line_t *L)
  {
    r3_t c;
    r3_cross(&(K->f), &(L->f), &c);
    return (hr2_point_t){c};
  }

hr2_point_t hr2_map_point(hr2_point_t *p, hr2_pmap_t *m)
  {
    r3_t q; 
    r3x3_map_row(&(p->c), &(m->dir), &q);
    return (hr2_point_t){q};
  }

hr2_line_t hr2_map_line(hr2_line_t *L, hr2_pmap_t *m)
  {
    r3_t f;
    r3x3_map_col(&(m->inv), &(L->f), &f);
    return (hr2_line_t){f};
  }

hr2_pmap_t hr2_comp_map(hr2_pmap_t *m, hr2_pmap_t *n)
  {
    r3x3_t dir, inv;
    r3x3_mul(&(m->dir), &(n->dir), &dir);
    r3x3_mul(&(n->inv), &(m->inv), &inv);
    return (hr2_pmap_t){dir, inv};
  }

r2_t hr2_line_dir(hr2_line_t *L)
  {
    double dx = L->f.c[2];
    double dy = -L->f.c[1];
    double length = hypot(dx, dy);
    return (r2_t){{dx/length, dy/length}};
  }

r2_t hr2_line_normal(hr2_line_t *L)
  {
    double nx = L->f.c[1];
    double ny = L->f.c[2];
    double length = hypot(nx, ny);
    return (r2_t){{nx/length, ny/length}};
  }

r2_t hr2_point_point_dir(hr2_point_t *frm, hr2_point_t *tto)
  {
    double fw = frm->c.c[0];
    double tw = tto->c.c[0];
    
    double fx = frm->c.c[1];
    double tx = tto->c.c[1];
    double dx = fw * tx - tw * fx;
    
    double fy = frm->c.c[2];
    double ty = tto->c.c[2];
    double dy = fw * ty - tw * fy;
    
    double length = hypot(dx, dy);
    return (r2_t){{dx/length, dy/length}};
  }

