/* See r2x2.h. */
/* Last edited on 2007-01-03 22:39:43 by stolfi */

#include <r2x2.h>
#include <r2.h>
#include <math.h>
#include <stdio.h>
#include <ioprotos.h>
#include <rmxn.h>

#define N 2

void r2x2_zero(r2x2_t *R)
  { R->c[0][0] = 0.0;
    R->c[0][1] = 0.0;
    R->c[1][0] = 0.0;
    R->c[1][1] = 0.0;
  }

void r2x2_ident(r2x2_t *R)
  { R->c[0][0] = 1.0;
    R->c[0][1] = 0.0;
    R->c[1][0] = 0.0;
    R->c[1][1] = 1.0;
  }

void r2x2_map_row (r2_t *x, r2x2_t *A, r2_t *r)
  { r2_t rr;
    rr.c[0] = x->c[0] * A->c[0][0] + x->c[1] * A->c[1][0];
    rr.c[1] = x->c[0] * A->c[0][1] + x->c[1] * A->c[1][1];
    
    r->c[0] = rr.c[0];
    r->c[1] = rr.c[1];
  }

void r2x2_map_col (r2x2_t *A, r2_t *x, r2_t *r)
  { r2_t rr;
    rr.c[0] = A->c[0][0] * x->c[0] + A->c[0][1] * x->c[1];
    rr.c[1] = A->c[1][0] * x->c[0] + A->c[1][1] * x->c[1];
    
    r->c[0] = rr.c[0];
    r->c[1] = rr.c[1];
  }

void r2x2_mul (r2x2_t *A, r2x2_t *B, r2x2_t *R)
  { r2x2_t RR;
    int i, j, k;
    for (i = 0; i < N; i++)
      for (j = 0; j < N; j++)
        { double s = 0.0;
          for (k = 0; k < N; k++) { s += A->c[i][k]*B->c[k][j]; }
          RR.c[i][j] = s;
        }
    for (i = 0; i < N; i++)
      for (j = 0; j < N; j++)
        { R->c[i][j] = RR.c[i][j]; }
  }

double r2x2_det (r2x2_t *A)
  { return A->c[0][0] * A->c[1][1] - A->c[1][0] * A->c[0][1]; }

void r2x2_adj (r2x2_t *A, r2x2_t *R)
  { r2x2_t RR;

    RR.c[0][0] =   A->c[1][1];
    RR.c[0][1] = - A->c[0][1];
    RR.c[1][0] = - A->c[1][0];
    RR.c[1][1] =   A->c[0][0];
    
    R->c[0][0] = RR.c[0][0];
    R->c[0][1] = RR.c[0][1];
    R->c[1][0] = RR.c[1][0];
    R->c[1][1] = RR.c[1][1];
  }

void r2x2_inv (r2x2_t *A, r2x2_t *R)
  { double d = A->c[0][0] * A->c[1][1] - A->c[1][0] * A->c[0][1];
    r2x2_t RR;

    RR.c[0][0] =   A->c[1][1]/d;
    RR.c[0][1] = - A->c[0][1]/d;
    RR.c[1][0] = - A->c[1][0]/d;
    RR.c[1][1] =   A->c[0][0]/d;
    
    R->c[0][0] = RR.c[0][0];
    R->c[0][1] = RR.c[0][1];
    R->c[1][0] = RR.c[1][0];
    R->c[1][1] = RR.c[1][1];
  }

void r2x2_print (FILE *f, r2x2_t *A)
  { r2x2_gen_print(f, A, NULL, NULL, NULL, NULL, NULL, NULL, NULL); }

void r2x2_gen_print 
  ( FILE *f, r2x2_t *A,
    char *fmt, 
    char *olp, char *osep, char *orp,
    char *ilp, char *isep, char *irp
  )
  { rmxn_gen_print(f, N, N, &(A->c[0][0]), fmt, olp, osep, orp, ilp, isep, irp); }  
