/* r2x2.h --- 2x2 matrices and operations on them */
/* Last edited on 2005-07-10 19:49:05 by stolfi */

#ifndef r2x2_H
#define r2x2_H

#include <stdio.h>
#include <r2.h>

typedef struct { double c[2][2]; } r2x2_t;

void r2x2_zero(r2x2_t *R);
  /* Stores in {R} the null matrix. */

void r2x2_ident(r2x2_t *R);
  /* Stores in {R} the identity matrix. */

void r2x2_map_row (r2_t *x, r2x2_t *A, r2_t *r);
  /* Sets {r} to the product of row vector {x} by matrix {A} */

void r2x2_map_col (r2x2_t *A, r2_t *x, r2_t *r);
  /* Sets {r} to the product of matrix {A} by column vector {x} */

void r2x2_mul (r2x2_t *A, r2x2_t *B, r2x2_t *R);
  /* Sets {R} to the product of matrices {A} and {B} */

double r2x2_det (r2x2_t *A);
  /* Returns the determinant of matrix {A} */

void r2x2_adj (r2x2_t *A, r2x2_t *R);
  /* Sets {R} to the adjoint of matrix {A} */

void r2x2_inv (r2x2_t *A, r2x2_t *R);
  /* Sets {R} to the inverse of matrix {A} */

void r2x2_print (FILE *f, r2x2_t *A);
  /* Prints matrix {A} to file {f}, with default format. */

void r2x2_gen_print
  ( FILE *f, r2x2_t *A,
    char *fmt, 
    char *olp, char *osep, char *orp, /* Outer delimiters. */
    char *ilp, char *isep, char *irp  /* Inner delimiters. */
  );
  /* Prints matrix {A} to file {f}, using {fmt} for each element.
    The matrix is bounded by {olp} and {orp}, and rows are separated
    by {osep}.  Each row is bounded by {ilp} and {irp}, and elements
    are separated by {isep}. Defaults are provided for any of these
    strings which are NULL. */

#endif

