/* r3x3.h --- 3x3 matrices and operations on them */
/* Last edited on 2005-07-10 19:48:55 by stolfi */

#ifndef r3x3_H
#define r3x3_H

#include <stdio.h>
#include <r3.h>

typedef struct { double c[3][3]; } r3x3_t;

void r3x3_zero(r3x3_t *R);
  /* Stores in {R} the null matrix. */

void r3x3_ident(r3x3_t *R);
  /* Stores in {R} the identity matrix. */

void r3x3_map_row (r3_t *x, r3x3_t *A, r3_t *r);
  /* Sets {r} to the product of row vector {x} by matrix {A} */

void r3x3_map_col (r3x3_t *A, r3_t *x, r3_t *r);
  /* Sets {r} to the product of matrix {A} by column vector {x} */

void r3x3_mul (r3x3_t *A, r3x3_t *B, r3x3_t *R);
  /* Sets {R} to the product of matrices {A} and {B} */

double r3x3_det (r3x3_t *A);
  /* Returns the determinant of matrix {A} */

void r3x3_adj (r3x3_t *A, r3x3_t *R);
  /* Sets {R} to the adjoint of matrix {A} */

void r3x3_inv (r3x3_t *A, r3x3_t *R);
  /* Sets {R} to the inverse of matrix {A} */

void r3x3_print (FILE *f, r3x3_t *A);
  /* Prints matrix {A} to file {f}, with default format. */

void r3x3_gen_print
  ( FILE *f, r3x3_t *A,
    char *fmt, 
    char *olp, char *osep, char *orp, /* Outer delimiters. */
    char *ilp, char *isep, char *irp  /* Inner delimiters. */
  );
  /* Prints matrix {A} to file {f}, using {fmt} for each element.
    The matrix is bounded by {olp} and {orp}, and rows are separated
    by {osep}.  Each row is bounded by {ilp} and {irp}, and elements
    are separated by {isep}. Defaults are provided for any of these
    strings which are NULL. */

#endif

