/* r4x4.h --- 4x4 matrices and operations on them */
/* Last edited on 2005-07-10 19:51:43 by stolfi */

#ifndef r4x4_H
#define r4x4_H

#include <stdio.h>
#include <r4.h>

typedef struct { double c[4][4]; } r4x4_t;

void r4x4_zero(r4x4_t *R);
  /* Stores in {R} the null matrix. */

void r4x4_ident(r4x4_t *R);
  /* Stores in {R} the identity matrix. */

void r4x4_map_row (r4_t *x, r4x4_t *A, r4_t *r);
  /* Sets {r} to the product of row vector {x} by matrix {A} */

void r4x4_map_col (r4x4_t *A, r4_t *x, r4_t *r);
  /* Sets {r} to the product of matrix {A} by column vector {x} */

void r4x4_mul (r4x4_t *A, r4x4_t *B, r4x4_t *R);
  /* Sets {R} to the product of matrices {A} and {B} */

double r4x4_det (r4x4_t *A);
  /* Returns the determinant of matrix {A} */

void r4x4_adj (r4x4_t *A, r4x4_t *R);
  /* Sets {R} to the adjoint of matrix {A} */

void r4x4_inv (r4x4_t *A, r4x4_t *R);
  /* Sets {R} to the inverse of matrix {A} */

void r4x4_print (FILE *f, r4x4_t *A);
  /* Prints matrix {A} to file {f}, with default format. */

void r4x4_gen_print
  ( FILE *f, r4x4_t *A,
    char *fmt, 
    char *olp, char *osep, char *orp, /* Outer delimiters. */
    char *ilp, char *isep, char *irp  /* Inner delimiters. */
  );
  /* Prints matrix {A} to file {f}, using {fmt} for each element.
    The matrix is bounded by {olp} and {orp}, and rows are separated
    by {osep}.  Each row is bounded by {ilp} and {irp}, and elements
    are separated by {isep}. Defaults are provided for any of these
    strings which are NULL. */

#endif

