/* r6x6.h --- 4x4 matrices and operations on them */
/* Last edited on 2003-11-02 19:02:19 by stolfi */

#ifndef r6x6_H
#define r6x6_H

#include <stdio.h>
#include <r6.h>

typedef struct { double c[6][6]; } r6x6_t;

void r6x6_zero(r6x6_t *R);
  /* Stores in {R} the null matrix. */

void r6x6_ident(r6x6_t *R);
  /* Stores in {R} the identity matrix. */

void r6x6_map_row (r6_t *x, r6x6_t *A, r6_t *R);
  /* Sets {R} to the product of row vector {x} by matrix {A} */

void r6x6_map_col (r6x6_t *A, r6_t *x, r6_t *R);
  /* Sets {R} to the product of matrix {A} by column vector {x} */

void r6x6_mul (r6x6_t *A, r6x6_t *B, r6x6_t *R);
  /* Sets {R} to the product of matrices {A} and {B} */

double r6x6_det (r6x6_t *A);
  /* Returns the determinant of matrix {A} */

void r6x6_adj (r6x6_t *A, r6x6_t *R);
  /* Sets {R} to the adjoint of matrix {A} */

void r6x6_inv (r6x6_t *A, r6x6_t *R);
  /* Sets {R} to the inverse of matrix {A} */

void r6x6_print (FILE *f, r6x6_t *A);
  /* Prints matrix {A} to file {f}, with default format. */

void r6x6_gen_print
  ( FILE *f, r6x6_t *A,
    char *fmt, 
    char *olp, char *osep, char *orp, /* Outer delimiters. */
    char *ilp, char *isep, char *irp  /* Inner delimiters. */
  );
  /* Prints matrix {A} to file {f}, using {fmt} for each element.
    The matrix is bounded by {olp} and {orp}, and rows are separated
    by {osep}.  Each row is bounded by {ilp} and {irp}, and elements
    are separated by {isep}. Defaults are provided for any of these
    strings which are NULL. */

#endif

