/* See {ia_trapez.h} */
/* Last edited on 2007-02-15 22:44:41 by stolfi */

#include <ia_trapez.h>

#include <affirm.h>
#include <flt.h>
#include <ia.h>
#include <ps.h>

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

ia_trapez_t ia_trapez_from_box(Interval *xr, Interval *yr)
  { return (ia_trapez_t) { *xr, *yr, *yr }; }

ia_trapez_t ia_trapez_from_ia_diff(Interval *xr, Interval *yr, Interval *dyr, int which)
  { Float dx = xr->hi - xr->lo;
    if (which == 1) { dx = -dx; }
    Interval yo = ia_add(*yr, ia_scale(*dyr, dx, One));
    if (which == 0)
      { return (ia_trapez_t){ *xr, *yr, yo }; }
    else
      { return (ia_trapez_t){ *xr, yo, *yr }; }
  }

void ia_trapez_print(FILE *wr, ia_trapez_t *tr)
  { int full = ia_is_full(&(tr->x));
    putc('[', wr);
    putc(' ', wr);
    ia_print_bound(wr, tr->x.lo, 0, full);
    fputs(" : ", wr);
    ia_print(wr, tr->yxlo);
    fputs(" __ ", wr);
    ia_print_bound(wr, tr->x.hi, 1, full);
    fputs(" : ", wr);
    ia_print(wr, tr->yxhi);
    putc(' ', wr);
    putc(']', wr);
  }

void ia_trapez_fill(FILE *pplot, Interval *yr, ia_trapez_t *tr, float cr, float cg, float cb)
  { int is_full = ia_is_full(&(tr->yxlo)) || ia_is_full(&(tr->yxhi));
    ROUND_NEAR;
    double xp[4], yp[4];
    xp[0] = tr->x.lo;   yp[0] = (is_full ? yr->lo : tr->yxlo.lo);
    xp[1] = tr->x.hi;   yp[1] = (is_full ? yr->lo : tr->yxhi.lo);
    xp[2] = tr->x.hi;   yp[2] = (is_full ? yr->hi : tr->yxhi.hi);
    xp[3] = tr->x.lo;   yp[3] = (is_full ? yr->hi : tr->yxlo.hi);
    ps_fill_polygon(pplot, xp, yp, 4, cr,cg,cb); 
  }    

void ia_trapez_draw(FILE *pplot, Interval *yr, ia_trapez_t *tr)
  { int is_full = ia_is_full(&(tr->yxlo)) || ia_is_full(&(tr->yxhi));
    ROUND_NEAR;
    double xp[4], yp[4];
    xp[0] = tr->x.lo;   yp[0] = (is_full ? yr->lo : tr->yxlo.lo);
    xp[1] = tr->x.hi;   yp[1] = (is_full ? yr->lo : tr->yxhi.lo);
    xp[2] = tr->x.hi;   yp[2] = (is_full ? yr->hi : tr->yxhi.hi);
    xp[3] = tr->x.lo;   yp[3] = (is_full ? yr->hi : tr->yxlo.hi);
    ps_draw_polygon(pplot, xp, yp, 4); 
  }
