/* See vec.h */
/* Last edited on 2007-01-31 20:11:53 by stolfi */

#include <vec.h>
#include <stdlib.h>
#include <stdint.h>
#include <affirm.h>

vec_t vec_new(int nel, size_t elsz)
  { vec_t v;
    v.nel = nel;
    v.el = (nel == 0 ? NULL : malloc(nel*elsz));
    affirm((nel == 0) || (v.el != NULL), "out of mem");
    return v;
  }

void vec_expand(vec_t *v, unsigned int index, size_t elsz)
  { if (index >= v->nel)
      { int nel = v->nel + index + 1;
        if (nel == 0) 
          { affirm(v->el == NULL, "bad elem pointer"); } 
        else
          { v->el = realloc(v->el, nel*elsz);
            affirm(v->el != NULL, "out of mem");
          }
        v->nel = nel;
      }
  }

void vec_trim(vec_t *v, unsigned int nel, size_t elsz)
  { if (nel != v->nel)
      { if (nel == 0) 
          { free(v->el); v->el = NULL; }
        else
          { v->el = realloc(v->el, nel*elsz);
            affirm(v->el != NULL, "out of mem");
          }
        v->nel = nel;
      }
  }
  
vec_t *vec_cast_ref(void *v)
  { return (vec_t *)v; }

vec_sub_t vec_as_sub(vec_t *v)
  { vec_sub_t vs;
    vs.nel = v->nel; vs.el = v->el; vs.step = 1;
    return vs;
  }
    
vec_sub_t vec_sub(vec_sub_t *vs, unsigned int start, int step, unsigned int nel, size_t elsz)
  { vec_sub_t rs;
    demand(step != 0, "step cannot be zero");
    if (start >= vs->nel)
      { rs.nel = 0; }
    else
      { int maxnel = (step > 0 ? (vs->nel - start)/step : (start-1)/(-step) + 1);
        rs.nel = (nel <= maxnel ? nel : maxnel);
      }
    if (rs.nel == 0) 
      { rs.el = NULL; rs.step = 1; }
    else
      { rs.el = (void *)((char *)vs->el + elsz*(start*vs->step));
        rs.step = step*vs->step;
      }
    return rs;
  }

/* SOME USEFUL TYPED VECTORS */

vec_typeimpl(int_vec_t,    int_vec,    int);
vec_typeimpl(uint_vec_t,   uint_vec,   unsigned int);
vec_typeimpl(double_vec_t, double_vec, double);
vec_typeimpl(bool_vec_t,   bool_vec,   bool_t);
vec_typeimpl(char_vec_t,   char_vec,   char);
vec_typeimpl(ubyte_vec_t,  ubyte_vec,  uint8_t);
vec_typeimpl(sbyte_vec_t,  sbyte_vec,  int8_t);
vec_typeimpl(string_vec_t, string_vec, char*);
vec_typeimpl(ref_vec_t,    ref_vec,    void*);


