/* Private definitions for pswr.h. */
/* Last edited on 2003-09-30 06:12:04 by stolfi */

#ifndef pswr_def_H
#define pswr_def_H

#include <stdio.h>

typedef struct PSStreamPrivate  /* A PSStream is a Postscript file plus some state: */
  { int verbose;           /* TRUE prints a trace of some commands. */
    /* Stream parameters: */ 
    int eps;            /* Kind of Postscript file (TRUE = encapsulated). */
    char *name;         /* File name prefix (no extension). */
    /* Page dimensions: */
    char *paperSize;    /* Paper size ({letter}, {a3}, etc.) or NULL. */
    double hPageSize;   /* Page width (in pt). */
    double vPageSize;   /* Page height (in pt). */
    /* Page layout (picture slot dimensions and counts): */
    double hFirst;      /* Left X coord of first slot. */
    double vFirst;      /* Top Y coord of first slot. */
    double hPicSize;    /* Picture width (in pt). */
    double vPicSize;    /* Picture height (in pt). */
    double hPicMargin;  /* Left/right margin for each picture (in pt). */
    double vPicMargin;  /* Top/bottom margin for each picture (in pt). */
    int captionLines;   /* Number of caption lines (including pic name). */
    int hCount;         /* Number of pictures in each row. */
    int vCount;         /* Number of pictures in each column. */
    /* Derived parameters: */
    double hSlotSize;   /* Total picture slot width, incl. margins. */
    double vSlotSize;   /* Total picture slot height, incl. margins and caption. */
    /* Pagination state: */
    FILE *file;         /* Current PS file, or NULL if none. */
    int curPage;        /* Current page number. */
    int curSlot;        /* Index of current picture slot in page. */
    /* Local graphics state (not available in Postscript file): */
    double xmin, xmax;     /* Client X plotting range (in client coords). */
    double ymin, ymax;     /* Client Y plotting range (in client coords). */
    double xscale, yscale; /* Scale factors (actual_pt/client_unit). */
    double fillColor[3];   /* Fill color for subsequent graphics ops. */
    int npages;            /* Number of pages already started (non-EPS). */
    int pagestarted;       /* Called pswr_begin_page w/o pswr_end_page (non-EPS). */
    /* These values mirror Postscript variables: */
    double hmin, hmax;     /* Actual X plotting range (in pt, page coords). */
    double vmin, vmax;     /* Actual Y plotting range (in pt, page coords). */
    int xn, yn;            /* Number of grid cells in each axis. */
    /* Font list: */
    int nfonts;            /* Number of fonts used in file. */
    char **fonts;          /* Fonts used (alloc size is {2^ceil(log2(nfonts))}. */
  } PSStreamPrivate;
  /* The fields {hPageSize,vPageSize} give the total dimensions
    of the physical page, or of the Encapsulated Postscript
    figure (virtual page).
    
    Individual pictures are arranged in each page (real or virtual) as
    a grid of {hCount} by {vCount} rectangular slots. Each slot
    measures {hSlotSize} by {vSlotSize} points, and comprises: a
    plottable area measuring {hPicSize} by {vPicSize}, blank margins
    of width {hPicMargin} and {vPicMargin} all around, plus enough
    space below the picture for {captionLines} lines of 10pt caption
    text.
    
    Slots are numbered from 0 to {hCount*vCount-1}, in Western reading
    order. After each call to {pswr_begin_picture}, the current picture slot
    is the one with index {curSlot} in the page {curPage}.
    
    If {curSlot} is -1, then a new page has been started, but no 
    {pswr_begin_picture} has been issued on it. The plot window is 
    set to the whole page. */

#endif
