#define PROG_NAME "makeshape"
#define PROG_DESC "creates some 2D maps using the quad-edge structure"
#define PROG_VERS "1.0"

/* Last edited on 2007-02-05 10:24:24 by stolfi */ 

#define PROG_COPYRIGHT \
  "Copyright  2007  State University of Campinas (UNICAMP)"
  
#define PROG_AUTH \
  "Modula-3 version {MakeShape.m3} created by Rober M. Rosi"\
  " and J. Stolfi, ca. 1994."
  
#define PROG_HIST \
  "Converted to C by J. Stolfi in fev/2007."
  
#define PROG_HELP \
  "  " PROG_NAME " \\\n" \
  "    -grid_order {NUM} \\\n" \
  "    -shape { torus | klein | ... | star }"

#define _GNU_SOURCE
#include <oct.h>
#include <bool.h>
#include <oct_shapes.h>
#include <frgb.h>
#include <argparser.h>
#include <jsfile.h>

#define _GNU_SOURCE
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

typedef enum 
  { shape_torus, 
    shape_bitorus,  
    shape_tritorus,  
    shape_klein,  
    shape_klein2,  
    shape_klein3,  
    shape_projective, 
    shape_tetra,  
    shape_stick,  
    shape_ring,  
    shape_cube,  
    shape_sausage,  
    shape_orange,  
    shape_fork,  
    shape_star
  } shape_t;

typedef struct options_t
  { shape_t shape;
    char *shape_name;
    char *output;
  } options_t;

int main(int argc, char **argv);
options_t *get_options (int argc, char **argv);
void putwr (oct_arc_t a);
oct_arc_t make_map(shape_t shape);
void write_map(char *prefix, oct_arc_t a);

/* Basic tools: */

oct_arc_t make_map(shape_t shape)
  { switch(shape) 
      { case shape_torus:       return make_torus();
        case shape_bitorus:     return make_bitorus();
        case shape_tritorus:    return make_tritorus();
        case shape_klein:       return make_klein();
        case shape_klein2:      return make_klein2();
        case shape_klein3:      return make_klein3();
        case shape_projective:  return make_projetive();
        case shape_tetra:       return make_tetra();
        case shape_stick:       return make_stick();
        case shape_ring:        return make_ring(5);
        case shape_cube:        return make_cube();
        case shape_sausage:     return make_sausage(3);
        case shape_orange:      return make_fork(7, 0);
        case shape_fork:        return make_fork(3, 2);
        case shape_star:        return make_fork(5, 1);
        default: assert(FALSE); return NULL;
      }
  } 
  
 int main(int argc, char **argv)
  { options_t *o = get_options(argc, argv);
    oct_arc_t m = make_map(o->shape);
    write_map(o->output, m);
    return 0;
  }
    
void write_map(char *prefix, oct_arc_t a)
  { char *filename = NULL;
    asprintf(&filename, "%s.qe", prefix);
    FILE *wr = open_write(filename, TRUE);
    oct_write_map(wr, a, NULL);
    fclose(wr);
    free(filename);
  }

#define PROG_INFO \
  "NAME\n" \
  "  " PROG_NAME " - " PROG_DESC "\n" \
  "\n" \
  "SYNOPSIS\n" \
  PROG_HELP "\n" \
  "\n" \
  "DESCRIPTION\n" \
  "  The program creates quad-edge descriptions of some 2D maps\n" \
  " on orientable and non-orientable surfaces, and writes them\n" \
  " to disk.\n" \
  "\n" \
  "OPTIONS\n" \
  "  -shape {NAME}\n" \
  "    Specifies which basic shape is to be built.\n" \
  "\n" \
  "  -grid-order {ORDER}\n" \
  "    Specifies the linear refinement factor to apply.\n" \
  "\n" \
  "DOCUMENTATION OPTIONS\n" \
  argparser_help_info_HELP_INFO "\n" \
  "\n" \
  "SEE ALSO\n" \
  "  ls(1).\n" \
  "\n" \
  "AUTHOR\n" \
  "  " PROG_AUTH ".\n" \
  "\n" \
  "MODIFICATION HISTORY\n" \
  "  " PROG_HIST ".\n" \
  "\n" \
  "WARRANTY\n" \
  argparser_help_info_NO_WARRANTY "\n" \
  "\n" \
  "RIGHTS\n" \
  "  " PROG_COPYRIGHT ".\n" \
  "\n" \
  argparser_help_info_STANDARD_RIGHTS

options_t *get_options (int argc, char **argv)
  {
    /* Initialize argument parser: */
    argparser_t *pp = argparser_new(stderr, argc, argv);
    argparser_set_help(pp, PROG_NAME " version " PROG_VERS ", usage:\n" PROG_HELP);
    argparser_set_info(pp, PROG_INFO);
    argparser_process_help_info_options(pp);
    
    /* Allocate the command line argument record: */
    options_t *o = (options_t *)notnull(malloc(sizeof(options_t)), "no mem"); 
    
    /* Parse keyword parameters: */

    argparser_get_keyword(pp, "-shape");  
    o->shape_name = argparser_get_next(pp);

    argparser_get_keyword(pp, "-output");  
    o->output = argparser_get_next(pp);

    if (0 == strcmp(o->shape_name, "torus")) 
      { o->shape = shape_torus; }
    else if (0 == strcmp(o->shape_name, "biTorus")) 
      { o->shape = shape_bitorus; }
    else if (0 == strcmp(o->shape_name, "triTorus")) 
      { o->shape = shape_tritorus; }
    else if (0 == strcmp(o->shape_name, "klein")) 
      { o->shape = shape_klein; }
    else if (0 == strcmp(o->shape_name, "klein2")) 
      { o->shape = shape_klein2; }
    else if (0 == strcmp(o->shape_name, "klein3")) 
      { o->shape = shape_klein3; }
    else if (0 == strcmp(o->shape_name, "projective")) 
      { o->shape = shape_projective; }
    else if (0 == strcmp(o->shape_name, "tetra")) 
      { o->shape = shape_tetra; }
    else if (0 == strcmp(o->shape_name, "stick")) 
      { o->shape = shape_stick; }
    else if (0 == strcmp(o->shape_name, "ring")) 
      { o->shape = shape_ring; }
    else if (0 == strcmp(o->shape_name, "cube")) 
      { o->shape = shape_cube; }
    else if (0 == strcmp(o->shape_name, "sausage")) 
      { o->shape = shape_sausage; }
    else if (0 == strcmp(o->shape_name, "orange")) 
      { o->shape = shape_orange; }
    else if (0 == strcmp(o->shape_name, "fork")) 
      { o->shape = shape_fork; }
    else if (0 == strcmp(o->shape_name, "star")) 
      { o->shape = shape_star; }
    else
      { argparser_error(pp, "unknown shape name\n"); }

    /* Parse positional arguments: */
    argparser_skip_parsed(pp);
    
    /* Check for spurious arguments: */
    argparser_finish(pp);
        
    return o;
  }
