/* 
 * File:   CustomFloat.hpp
 * Author: Mario J. Martin <dominonurbs$gmail.com>
 *
 * Class to define floating point types with custom precision for error noise analisis
 */

#ifndef _CUSTOMFLOAT_DEBUG_HPP
#define	_CUSTOMFLOAT_DEBUG_HPP

#include <iostream>
#include <math.h>
#include "cf_definitions.hpp"

#include "cfToString.cpp"
#include "cfConvert.cpp"
#include "cfBinarySwap.cpp"
#include "cfBinaryAddSub.cpp"
#include "cfChangeSign.cpp"
#include "cfAddition.cpp"
#include "cfSubtraction.cpp"
#include "cfMultiplication.cpp"
#include "cfDivision.cpp"
#include "cfMath.cpp"
#include "cfComparison.cpp"

/* MACROS */


int num_operations = 0;

/* GLOBAL CALLS */
template< unsigned int M, unsigned int E >
void check(double f, const CustomFloat< M, E >& r)
{
	double fr = (double)r;
	if (abs(fr - f) > 0.001){
		return false;
	}
	num_operations++;

	return true;
}

void check(bool f, bool r)
{
	if (f != r){
		return false;
	}

	return true;
}

/** power = u ^ v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > abs (const CustomFloat< M, E >& u)
{
	if (check(abs((double)u), cfAbs(u)){
		_cf_handle_error_("\n***Result does not match!***");
	}
	return cfAbs(u);
}

template< unsigned int M, unsigned int E >
const CustomFloat< M, E > pow (const CustomFloat< M, E >& u, const int d)
{
	if (check(pow((double)u, d), cfPow(u, d)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfPow(u, d);
}


/** Square root */
template<int M, int E >
CustomFloat< M, E > sqrt(const CustomFloat< M, E >& y)
{
	if (check(sqrt((double)y), cfSqrt(y)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfSqrt(y);
}


/** sin(x) */
template<int M, int E >
CustomFloat< M, E > sin(const CustomFloat< M, E >& y)
{
	if (check(sin((double)y), cfSin(y)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfSin(y);
}


/** cos(x) */
template<int M, int E >
CustomFloat< M, E > cos(const CustomFloat< M, E >& y)
{
	if (check(cos((double)y), cfCos(y)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfCos(y);
}


/* GLOBAL OPERATOR OVERLOADING */
/** Add operation \f${lvalue} = u + v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator+ (const CustomFloat< M, E >& u, const CustomFloat< M, E >& v)
{
	if (check((double)u + (double)v, cfAdd(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfAdd(u, v);
}


/** Add operation \f${lvalue} = u + v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator+ (const CustomFloat< M, E >& u, const double d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u + d, cfAdd(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfAdd(u, v);
}

/** Add operation \f${lvalue} = u + v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator+ (const CustomFloat< M, E >& u, const float d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u + d, cfAdd(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfAdd(u, v);
}

/** Add operation \f${lvalue} = u + v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator+ (const CustomFloat< M, E >& u, const int d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u + d, cfAdd(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfAdd(u, v);
}

/** Add operation \f${lvalue} = u + v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator+ (const double d, const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > v = d;
	if (check((double)u + d, cfAdd(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfAdd(u, v);
}

/** Add operation \f${lvalue} = u + v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator+ (const float d, const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > v = d;
	if (check((double)u + d, cfAdd(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfAdd(u, v);
}

/** Add operation \f${lvalue} = u + v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator+ (const int d, const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > v = d;
	if (check((double)u + d, cfAdd(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfAdd(u, v);
}

/** Substract operation \f${lvalue} = u - v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator- (const CustomFloat< M, E >& u, const CustomFloat< M, E >& v)
{
	if (check((double)u - (double)v, cfSub(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfSub(u, v);
}


/** Substract operation \f${lvalue} = u - v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator- (const CustomFloat< M, E >& u, const double d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u - d, cfSub(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfSub(u, v);
}

/** Substract operation \f${lvalue} = u - v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator- (const CustomFloat< M, E >& u, const float d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u - d, cfSub(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfSub(u, v);
}

/** Substract operation \f${lvalue} = u - v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator- (const CustomFloat< M, E >& u, const int d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u - d, cfSub(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfSub(u, v);
}

template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator- (const double d, const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > v = d;
	if (check(d - (double)u, cfSub(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfSub(v, u);
}

/** Substract operation \f${lvalue} = u - v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator- (const float d, const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > v = d;
	if (check(d - (double)u, cfSub(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfSub(v, u);
}

/** Substract operation \f${lvalue} = u - v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator- (const int d, const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > v = d;
	if (check(d - (double)u, cfSub(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfSub(v, u);
}

/** product operation \f${lvalue} = u * v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator* (const CustomFloat< M, E >& u, const CustomFloat< M, E >& v)
{
	if (check((double)u * (double)v, cfMul(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfMul(u, v);
}


/** product operation \f${lvalue} = u * v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator* (const CustomFloat< M, E >& u, const double d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u * d, cfMul(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfMul(u, v);
}

/** product operation \f${lvalue} = u * v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator* (const CustomFloat< M, E >& u, const float d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u * d, cfMul(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfMul(u, v);
}

/** product operation \f${lvalue} = u * v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator* (const CustomFloat< M, E >& u, const int d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u * d, cfMul(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfMul(u, v);
}

/** product operation \f${lvalue} = u * v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator* (const double d, const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > v = d;
	if (check((double)u * d, cfMul(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfMul(u, v);
}

/** product operation \f${lvalue} = u * v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator* (const float d, const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > v = d;
	if (check((double)u * d, cfMul(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfMul(u, v);
}

/** product operation \f${lvalue} = u * v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator* (const int d, const CustomFloat< M, E >& u)
{
	// a little performance improvement
	if (d == 2 && (u.m & CustomFloat< M, E >::MASK_MANTISSA) != 0){
		return CustomFloat< M, E >(u.m, u.e - 1);
	}

	CustomFloat< M, E > v = d;
	if (check((double)u * d, cfMul(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfMul(u, v);
}

/** division operation \f${lvalue} = u / v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator/ (const CustomFloat< M, E >& u, const CustomFloat< M, E >& v)
{
	if (check((double)u / (double)v, cfDiv(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfDiv(u, v);
}


/** division operation \f${lvalue} = u / v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator/ (const CustomFloat< M, E >& u, const double d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u / d, cfDiv(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfDiv(u, v);
}

/** division operation \f${lvalue} = u / v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator/ (const CustomFloat< M, E >& u, const float d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u / d, cfDiv(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfDiv(u, v);
}

/** division operation \f${lvalue} = u / v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator/ (const CustomFloat< M, E >& u, const int d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u / d, cfDiv(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfDiv(u, v);
}

/** division operation \f${lvalue} = u / v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator/ (const double d, const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > v = d;
	if (check(d / (double)u, cfDiv(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfDiv(v, u);
}

/** division operation \f${lvalue} = u / v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator/ (const float d, const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > v = d;
	if (check(d / (double)u, cfDiv(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfDiv(v, u);
}

/** division operation \f${lvalue} = u / v\f$ */
template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator/ (const int d, const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > v = d;
	if (check(d / (double)u, cfDiv(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfDiv(v, u);
}

/** Comparison operation */
template< unsigned int M, unsigned int E >
const bool operator< (const CustomFloat< M, E >& u, const CustomFloat< M, E >& v)
{
	if (check((double)u < (double)v, cfLowerThan(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfLowerThan(u, v);
}

template< unsigned int M, unsigned int E >
const bool operator< (const CustomFloat< M, E >& u, const double d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u < d, cfLowerThan(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfLowerThan(u, v);
}

template< unsigned int M, unsigned int E >
const bool operator< (const CustomFloat< M, E >& u, const float d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u < d, cfLowerThan(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfLowerThan(u, v);
}

template< unsigned int M, unsigned int E >
const bool operator< (const CustomFloat< M, E >& u, const int d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u < d, cfLowerThan(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfLowerThan(u, v);
}

template< unsigned int M, unsigned int E >
const bool operator< (const double d, const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > v = d;
	if (check(d < (double)u, cfLowerThan(v, u)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfLowerThan(v, u);
}

template< unsigned int M, unsigned int E >
const bool operator< (const float d, const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > v = d;
	if (check(d < (double)u, cfLowerThan(v, u)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfLowerThan(v, u);
}

template< unsigned int M, unsigned int E >
const bool operator< (const int d, const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > v = d;
	if (check(d < (double)u, cfLowerThan(v, u)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfLowerThan(v, u);
}

template< unsigned int M, unsigned int E >
const bool operator> (const CustomFloat< M, E >& u, const CustomFloat< M, E >& v)
{
	if (check((double)u > (double)v, cfGreaterThan(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfGreaterThan(u, v);
}

template< unsigned int M, unsigned int E >
const bool operator> (const CustomFloat< M, E >& u, const double d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u > d, cfGreaterThan(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfGreaterThan(u, v);
}

template< unsigned int M, unsigned int E >
const bool operator> (const CustomFloat< M, E >& u, const float d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u > d, cfGreaterThan(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfGreaterThan(u, v);
}

template< unsigned int M, unsigned int E >
const bool operator> (const CustomFloat< M, E >& u, const int d)
{
	CustomFloat< M, E > v = d;
	if (check((double)u > d, cfGreaterThan(u, v)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfGreaterThan(u, v);
}

template< unsigned int M, unsigned int E >
const bool operator> (const double d, const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > v = d;
	if (check(d > (double)u, cfGreaterThan(v, u)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfGreaterThan(v, u);
}

template< unsigned int M, unsigned int E >
const bool operator> (const float d, const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > v = d;
	if (check(d > (double)u, cfGreaterThan(v, u)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfGreaterThan(v, u);
}

template< unsigned int M, unsigned int E >
const bool operator> (const int d, const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > v = d;
	if (check(d > (double)u, cfGreaterThan(v, u)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfGreaterThan(v, u);
}

template< unsigned int M, unsigned int E >
const CustomFloat< M, E > operator- (const CustomFloat< M, E >& u)
{
	if (check( -((double)u), cfChangeSign(u)){
		_cf_handle_error_("\n***Result does not match!***");
	}

	return cfChangeSign(u);
}


template < unsigned int M, unsigned int E >
class CustomFloat {
	
public:
    /* DATA */
    man_t m;    /// mantissa of the number
    exp_t e;    /// exponent of the number


    /* CONSTRUCTOR */
    /** Default constructor */
    CustomFloat()
	{
        this->m = 0;
        this->e = 0;
    }

    /** constructor */
    CustomFloat(const man_t m, const exp_t e)
	{
        this->m = m;
        this->e = e;
    }

    /** Copy constructor */
    CustomFloat(const CustomFloat& y)
	{
        this->m = y.m;
        this->e = y.e;
    }

	/** constructor */
    CustomFloat(const double value)
	{
        *this = cfConvert< M, E >(value);
    }

    /** Destructor */
    ~CustomFloat()
	{
        // nothing to be done
    }

    /** initilizer */
	const CustomFloat& operator() (const man_t m, const exp_t e)
	{
        this->m = m;
        this->e = e;
		return *this;
	}

	/** copy */
    const CustomFloat& operator() (const CustomFloat& y)
	{
        this->m = y.m;
        this->e = y.e;
		return *this;
    }

    /* UNARY OPERATORS */
    CustomFloat* operator &()
	{
		return this;
	}

    const CustomFloat& operator *();

    bool& operator [] (const int index);

	const CustomFloat& operator ++()
	{
		*this = cfAdd(*this, 1);
	}

	const CustomFloat& operator +()
	{
		// Nothing to be done
	}

	const CustomFloat operator -()
	{
		return cfChangeSign(*this);
	}

	const CustomFloat& operator --()
	{
		*this = cfSub(*this, 1);
	}

    /* ASIGMENT OPERATOR */

	const CustomFloat& operator =(const CustomFloat& u)
	{
		this->m = u.m;
		this->e = u.e;

		return *this;
	}

	const CustomFloat& operator =(const double value)
	{
		*this = cfConvert< M, E >(value);

		return *this;
	}

    const CustomFloat& operator +=(const CustomFloat& u)
	{
		*this = cfAdd(*this, u);
		return *this;
	}

    const CustomFloat& operator -=(const CustomFloat& u)
	{
		*this = cfSub(*this, u);
		return *this;
	}

    const CustomFloat& operator *=(const CustomFloat& u)
	{
		*this = cfMul(*this, u);
		return *this;
	}

    const CustomFloat& operator /=(const CustomFloat& u)
	{
		*this = cfDiv(*this, u);
		return *this;
	}

    /* CASTING */
    template< unsigned int MM, unsigned int EE > 
    operator CustomFloat<MM, EE>() const;

	operator const long() const
	{
		return (long)cfConvert(*this);
	}

	operator const int() const
	{
		return (int)cfConvert(*this);
	}

	operator const float() const
	{
		return (float)cfConvert(*this);		
	}

	operator const double() const
	{
		return cfConvert(*this);		
	}

    /* STATIC METHODS */

	/** Get the maximun value allowed by the representation */
	static double get_maximun_value()
	{
		
	}

	/* STATIC VARIABLES */
	static const man_t MASK_SIGN_BIT = (1 << M);
	static const man_t MASK_MANTISSA_MOST_SIGNIFICANT_BIT = (1 << (M - 1));
	static const man_t MASK_MANTISSA = ~(~0 << M);

	static const exp_t MASK_EXPONENT_MOST_SIGNIFICANT_BIT = (1 << (E - 1));
	static const exp_t MASK_EXPONENT = ~(~0 << E);
 };

/* ALIASES */


/* SPECIALIZATION */
// NaN
//CustomFloat<sizeof(man_t) * 4, sizeof(exp_t) * 4> cfNaN(~0,0);


#endif	/* _CUSTOMFLOAT_HPP */

