/* 
 * File:   cfAddition.cpp
 * Author: Mario J. Martin <dominonurbs$gmail.com>
 *
 */

#include "cf_definitions.hpp"


/** Adds two floating point variables with custom precission */
template< unsigned int M, unsigned int E >
static inline CustomFloat< M, E > cfAdd(const CustomFloat< M, E >& u, const CustomFloat< M, E >& v)
{
	CustomFloat< M, E > c;		// result
	register man_t ma, mb;		// mantissas
	bool sign_a, sign_b;		// signs

	ma = u.m & CustomFloat< M, E >::MASK_MANTISSA;
	mb = v.m & CustomFloat< M, E >::MASK_MANTISSA;

	// check the sign
	sign_a = (u.m & CustomFloat< M, E >::MASK_SIGN_BIT) == 0 ? false : true;
	sign_b = (v.m & CustomFloat< M, E >::MASK_SIGN_BIT) == 0 ? false : true;

	// special cases
	if (ma == 0)
		return v;
	if (mb == 0)
		return u;

	// Normalize to the highest exponent. The first significant bit is always 1, so we shift to have the same exponent. There is no rounding
	if (u.e > v.e){
		if (u.e - v.e >= M)
			return u;	// the second operand has drop bellow available precission
	
		mb >>= (u.e - v.e);
		c.e = u.e;
	}
	else if (u.e < v.e){
		if (v.e - u.e >= M)
			return v;		// the first operand has drop bellow available precission
				
		ma >>= (v.e - u.e);
		c.e = v.e;
	}
	else{
		c.e = u.e;
	}

	if (sign_a == false && sign_b == false){
		cfBinaryAdd(c, ma, mb);
	}
	else if (sign_a == true && sign_b == false){
		cfBinarySub(c, mb, ma);
	}
	else if (sign_a == false && sign_b == true){
		cfBinarySub(c, ma, mb);
	}
	else if (sign_a == true && sign_b == true){
		cfBinaryAdd(c, ma, mb);
		c.m |= CustomFloat< M, E >::MASK_SIGN_BIT;
	}

	return c;
}

