/* 
 * File:   cfBinaryAddSub.cpp
 * Author: Mario J. Martin <dominonurbs$gmail.com>
 *
 * Binary addition and subtraction of two unsigned numbers
 */

#include "cf_definitions.hpp"


// Adds two unsigned binary numbers
template< unsigned int M, unsigned int E >
static inline void cfBinaryAdd(CustomFloat< M, E >& c, const man_t a, const man_t b)
{
	// We prefer to do this operation unsigned
	c.m = a + b;
	if ((c.m > CustomFloat< M, E >::MASK_MANTISSA)){
		c.m >>= 1;
		c.e++;
	} 

#ifdef CF_CHECK_OVERFLOW 
	// check overflow
	if (c.e >= CustomFloat< M, E >::MASK_EXPONENT){
		_cf_handle_error_ ("Overflow detected!");
		#ifdef CF_THROW_ERRORS
			throw("Overflow detected!");
		#endif
	}
#endif
}

// Substract two unsigned binary numbers a - b. 
template< unsigned int M, unsigned int E >
static inline void cfBinarySub(CustomFloat< M, E >& c, const man_t a, const man_t b)
{
	// special case
	if (a == b){
		c(0, 0);
		return;
	}

	// make the substraction
	if (a > b){
		c.m = a - b;
	}
	else{
		c.m = b - a;
	}

	// normalize
	while ( c.m <= (CustomFloat< M, E >::MASK_MANTISSA>> 1) ){
		c.m <<= 1;
		c.e--;
	}

#ifdef CF_CHECK_OVERFLOW 
	// check overflow
	if (c.e <= -CustomFloat< M, E >::MASK_EXPONENT){
		_cf_handle_error_ ("Overflow detected!");
		#ifdef CF_THROW_ERRORS 
			throw("Overflow detected!");
		#endif
	}
#endif

	// set the sign
	if (b > a){
		c.m |= CustomFloat< M, E >::MASK_SIGN_BIT;
	} 
}

