/* 
 * File:   cfAddition.cpp
 * Author: Mario J. Martin <dominonurbs$gmail.com>
 *
 */

#include "cf_definitions.hpp"


/** Compares two floating point variables with custom precission */
template< unsigned int M, unsigned int E >
static inline bool cfGreaterThan(const CustomFloat< M, E >& u, const CustomFloat< M, E >& v)
{
	bool sign_a, sign_b;		// signs
	man_t ma, mb;			// mantissas

	sign_a = (u.m & CustomFloat< M, E >::MASK_SIGN_BIT) == 0 ? false : true;
	sign_b = (v.m & CustomFloat< M, E >::MASK_SIGN_BIT) == 0 ? false : true;

	ma = u.m & CustomFloat< M, E >::MASK_MANTISSA;
	mb = v.m & CustomFloat< M, E >::MASK_MANTISSA;

	// special case
	if (ma == 0 && mb == 0){
		return false;
	}

	if (sign_a == false && sign_b == true){
	// u is positive and v is negative 
		return true;
	}
	else if (sign_a == true && sign_b == false){
	// u is negative and v is positive 
		return false;
	}
	else if (sign_a == true && sign_b == true){
		// both are negative 
		if (ma == 0){
			return true;
		}
		else if (mb == 0){
			return false;
		}
		else if (u.e > v.e){
			return false;
		}
		else if (u.e < v.e){
			return true;
		}
		else{
			if (u.m > v.m){
				return false;
			}
			else if (u.m < v.m){
				return true;
			}
			else{
				// are the same
				return false;
			}
		}
	}
	else{  //sign_a == false && sign_b == false
		// both are positive 
		if (ma == 0){
			return false;
		}
		else if (mb == 0){
			return true;
		}
		else if (u.e > v.e){
			return true;
		}
		else if (u.e < v.e){
			return false;
		}
		else{
			if (u.m > v.m){
				return true;
			}
			else if (u.m < v.m){
				return false;
			}
			else{
				// are the same
				return false;
			}
		}
	}
}

/** Compares two floating point variables with custom precission */
template< unsigned int M, unsigned int E >
static inline bool cfLowerThan(const CustomFloat< M, E >& u, const CustomFloat< M, E >& v)
{
	bool sign_a, sign_b;	// signs
	man_t ma, mb;			// mantissas

	sign_a = (u.m & CustomFloat< M, E >::MASK_SIGN_BIT) == 0 ? false : true;
	sign_b = (v.m & CustomFloat< M, E >::MASK_SIGN_BIT) == 0 ? false : true;

	ma = u.m & CustomFloat< M, E >::MASK_MANTISSA;
	mb = v.m & CustomFloat< M, E >::MASK_MANTISSA;

	// special case; both are zero.
	if (ma == 0 && mb == 0){
		return false;
	}

	if (sign_a == false && sign_b == true){
	// u is positive and v is negative 
		return false;
	}
	else if (sign_a == true && sign_b == false){
	// u is negative and v is positive 
		return true;
	}
	else if (sign_a == true && sign_b == true){
		// both are negative 

		if (ma == 0){
			return false;
		}
		else if (mb == 0){
			return true;
		}
		else if (u.e > v.e ){
			return true;
		}
		else if (u.e < v.e){
			return false;
		}
		else{
			if (u.m > v.m){
				return true;
			}
			else if (u.m < v.m){
				return false;
			}
			else{
				// are the same
				return false;
			}
		}
	}
	else{ //sign_a == false && sign_b == false
		// both are positive 
		if (ma == 0){
			return true;
		}
		else if (mb == 0){
			return false;
		}
		else if (u.e > v.e){
			return false;
		}
		else if (u.e < v.e){
			return true;
		}
		else{
			if (u.m > v.m){
				return false;
			}
			else if (u.m < v.m){
				return true;
			}
			else{
				// are the same
				return false;
			}
		}
	}
}

