/* 
 * File:   cfDivision.cpp
 * Author: Mario J. Martin <dominonurbs$gmail.com>
 *
 */

#include "cf_definitions.hpp"


/** Serial multiplication between two floating point variables with custom precission */
template< unsigned int M, unsigned int E >
static inline CustomFloat< M, E > cfDiv(const CustomFloat< M, E >& u, const CustomFloat< M, E >& v)
{
	CustomFloat< M, E > c;

	// get mantissas
	register man_t ma = (u.m & CustomFloat< M, E >::MASK_MANTISSA);
	register man_t mb = (v.m & CustomFloat< M, E >::MASK_MANTISSA);

	// check division by zero
	if (mb == 0){
		_cf_handle_error_("Division by zero!");
		#ifdef CF_THROW_ERRORS
			throw("Division by zero!");
		#endif
		// returns infinite
		if ( (u.m & CustomFloat< M, E >::MASK_SIGN_BIT) == (u.m & CustomFloat< M, E >::MASK_SIGN_BIT) )
			return CustomFloat< M, E > (CustomFloat< M, E >::MASK_MANTISSA, CustomFloat< M, E >::MASK_EXPONENT );
		else
			return CustomFloat< M, E > (CustomFloat< M, E >::MASK_MANTISSA | CustomFloat< M, E >::MASK_SIGN_BIT, CustomFloat< M, E >::MASK_EXPONENT );
	}

	// special case
	if (ma == 0)
		return CustomFloat< M, E > (0, 0);

	// set the exponent
	c.e = u.e - v.e - M + 1;

	if (ma < mb ){
		ma <<= 1;	// we ensure that ma >= mb but we "invade" the sign bit by doing this
		c.e--;
	}
	
	#ifdef CF_CHECK_OVERFLOW 
	// check overflow
	if (c.e >= CustomFloat< M, E >::MASK_EXPONENT || c.e  <= -CustomFloat< M, E >::MASK_EXPONENT){
		_cf_handle_error_ ("Overflow detected!");
		#ifdef CF_THROW_ERRORS
			throw("Overflow detected!");
		#endif
	}
	#endif

	// make the division
	for (int i = 0; i < M; i++){
		c.m <<= 1;
		if (ma >= mb){
			c.m |= 1;
			ma -= mb;
		}
		ma <<= 1;
	}
	
	// set the sign
	if ( (u.m & CustomFloat< M, E >::MASK_SIGN_BIT) != (v.m & CustomFloat< M, E >::MASK_SIGN_BIT) )
		c.m |= CustomFloat< M, E >::MASK_SIGN_BIT;
	
	return c;

}

