/* 
 * File:   cfMath.cpp
 * Author: Mario J. Martin <dominonurbs$gmail.com>
 *
 */

#include "cf_definitions.hpp"


static int cfFac(const int n){
	if (n == 0)
		return 1;

	int r = 1;
	for (int i = 2; i <= n; i++){
		r *= i;
	}

	return r;
}

/** abs(x) */
template< unsigned int M, unsigned int E >
static CustomFloat< M, E > cfAbs(const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > c;

	if ( (u.m & CustomFloat< M, E >::MASK_SIGN_BIT) != 0){
		c.m = u.m & CustomFloat< M, E >::MASK_MANTISSA;
	}
	else{
		c.m = u.m;
	}

	c.e = u.e;

	return c;
}

/** Power */
template< unsigned int M, unsigned int E >
static CustomFloat< M, E > cfPow(const CustomFloat< M, E >& u, const int k)
{
	CustomFloat< M, E > c;
	CustomFloat< M, E > b;

	if (k == 0){
		c = 1;
		return c;
	}
	if (k < 0){
		k = -k;
		b = 1 / u;
	}
	else{
		b = u;
	}

	c = b;
	for (int i = 1; i < k; i++){
		c *= b;
	}

	return c;
}


/** Trigonometric fuction */
template< unsigned int M, unsigned int E >
static inline CustomFloat< M, E > cfSin(const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > c = sin((double)u);

	return c;
}


/** Trigonometric fuction */
template< unsigned int M, unsigned int E >
static inline CustomFloat< M, E > cfCos(const CustomFloat< M, E >& u)
{
	CustomFloat< M, E > c = cos((double)u);

	return c;
}


/** Square root */
template< unsigned int M, unsigned int E >
static inline CustomFloat< M, E > cfSqrt(const CustomFloat< M, E >& u)
{
	// check if x == 0
	if ((u.m & CustomFloat< M, E >::MASK_MANTISSA) == 0)
		return CustomFloat< M, E >(0, 0);

	// check if x < 0
	if ((u.m & CustomFloat< M, E >::MASK_SIGN_BIT) != 0){
		_cf_handle_error_("negative square!");
		#ifdef CF_THROW_ERRORS
			throw("negative square!");
		#endif
	}

	CustomFloat< M, E > c = sqrt((double)u);

	return c;
}

