/* 
 * File:   cfMultiplication.cpp
 * Author: Mario J. Martin <dominonurbs$gmail.com>
 *
 */

#include "cf_definitions.hpp"


/** Serial multiplication between two floating point variables with custom precission */
template< unsigned int M, unsigned int E >
static inline CustomFloat< M, E > cfMul(const CustomFloat< M, E >& u, const CustomFloat< M, E >& v)
{
	register man_t ma = u.m & CustomFloat< M, E >::MASK_MANTISSA;
	register unsigned short i, shift, shift_norm;

	// check for zero
	if (ma == 0 || (v.m & CustomFloat< M, E >::MASK_MANTISSA) == 0){
		CustomFloat< M, E >(0, 0);
	}

	CustomFloat< M, E > c;
	c.e = u.e + v.e;

#ifdef CF_CHECK_OVERFLOW 
	// check overflow
	if (c.e >= CustomFloat< M, E >::MASK_EXPONENT){
		_cf_handle_error_ ("Overflow detected!");
		#ifdef CF_THROW_ERRORS
			throw("Overflow detected!");
		#endif
	}
#endif

	// check if the result will be below precission
	 if (c.e <= -CustomFloat< M, E >::MASK_EXPONENT)
		 return CustomFloat< M, E >(0, 0);

	// make the multiplication
	shift = 0;			// shift due multiplication operation
	shift_norm = 0;		// shift due normalization
	for (i = 0; i < M; i++){
		if (((v.m>>i) & 1) != 0){
			// shift
			c.m >>= shift;		// there is no guard bits or rounding of any kind here and the bits are lost.
			c.e += shift;
			shift = 1;
			// add
			c.m += (ma >> shift_norm);
			// normalize
			if ((c.m > CustomFloat< M, E >::MASK_MANTISSA)){
				c.m >>= 1;		// again there is no guard bits or round operations here.
				c.e++;
				shift_norm++;

				#ifdef CF_CHECK_OVERFLOW 
					// check overflow
					if (c.e >= CustomFloat< M, E >::MASK_EXPONENT){
						_cf_handle_error_ ("Overflow detected!");
						#ifdef CF_THROW_ERRORS
							throw("Overflow detected!");
						#endif
					}
				#endif

				// check if the result will be below precission
				if (c.e <= -CustomFloat< M, E >::MASK_EXPONENT)
					return CustomFloat< M, E >(0, 0);

			}
		}
		else{
			shift++;
		}
	}

	// set the sign
	if ( (u.m & CustomFloat< M, E >::MASK_SIGN_BIT) != (v.m & CustomFloat< M, E >::MASK_SIGN_BIT))
		c.m |= CustomFloat< M, E >::MASK_SIGN_BIT;

	return c;
}

