/* 
 * File:   cfSubstraction.cpp
 * Author: Mario J. Martin <dominonurbs$gmail.com>
 *
 */

#include "cf_definitions.hpp"


/** Substract two floating point variables with custom precission */
template< unsigned int M, unsigned int E >
static inline CustomFloat< M, E > cfSub(const CustomFloat< M, E >& u, const CustomFloat< M, E >& v)
{
	CustomFloat< M, E > c;		// result
	register man_t ma, mb;		// mantissas
	bool sign_a, sign_b;		// signs

	ma = u.m & CustomFloat< M, E >::MASK_MANTISSA;
	mb = v.m & CustomFloat< M, E >::MASK_MANTISSA;

	// special case
	if (mb == 0){
		return u;
	}

	// check the sign
	sign_a = (u.m & CustomFloat< M, E >::MASK_SIGN_BIT) == 0 ? false : true;
	sign_b = (v.m & CustomFloat< M, E >::MASK_SIGN_BIT) == 0 ? true : false;

	// special case
	if (ma == 0){
		c.e = v.e;
		if (sign_b == false){
			c.m = mb;
		}
		else{
			c.m = mb | CustomFloat< M, E >::MASK_SIGN_BIT;
		}

		return c;
	}

	// Normalize to the highest exponent. The first significant bit is always 1, so we shift one place. There is no rounding
	if (u.e > v.e){
		c.e = u.e;
		if ((u.e - v.e) >= M){
			// It drops bellow available precision.
			if (sign_a == false){
				c.m = ma;
			}
			else{
				c.m = ma | CustomFloat< M, E >::MASK_SIGN_BIT;
			}
			return c;
		}
		else{
			mb >>= (u.e - v.e);
		}
	}
	else{
		c.e = v.e;
		if ((v.e - u.e) >= M){
			// It drops bellow available precision.
			if (sign_b == false){
				c.m = mb;
			}
			else{
				c.m = mb | CustomFloat< M, E >::MASK_SIGN_BIT;
			}
			return c;
		}
		else{
			ma >>= (v.e - u.e);
		}
	}

	// special case
	if (ma == 0){
		c.e = v.e;
		if (sign_b == false){
			c.m = mb;
		}
		else{
			c.m = mb | CustomFloat< M, E >::MASK_SIGN_BIT;
		}

		return c;
	}

	if (sign_a == false && sign_b == false){
		cfBinaryAdd(c, ma, mb);
	}
	else if (sign_a == true && sign_b == false){
		cfBinarySub(c, mb, ma);
	}
	else if (sign_a == false && sign_b == true){
		cfBinarySub(c, ma, mb);
	}
	else if (sign_a == true && sign_b == true){
		cfBinaryAdd(c, ma, mb);
		c.m |= CustomFloat< M, E >::MASK_SIGN_BIT;
	}

	return c;
}

