/* 
 * File:   main.cpp
 * Author: Mario J. Martin <dominonurbs$gmail.com>
 *
 * Test framework for the Custom Float type
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "CustomFloat.hpp"

//#include "common/trace.c"
#define trace printf


// Two alternatives to create a new variableusing define or typedef. (I don't have arguments to decide which one is best; both works well).
typedef CustomFloat<24, 8>  cfloat;

//#define cfloat CustomFloat<4, 4>

#define _sign_(m) ((m & cfloat::MASK_SIGN_BIT) != 0) ? '-' : '+'
void test_swap()
{
	man_t g = 0xAF000009;
	trace("\n%s", cfToString(g));
	g = cfBinarySwap(g);
	trace("\n%s", cfToString(g));
} 

void test_conversion()
{
	trace("\n\n*** CONVERSION ***");
	cfloat a;
	a = 2147483647;
	trace("\na = (cfloat)%d ==> %c0x%x*2^%i ~ %g", 2147483647, _sign_(a.m), a.m, a.e, (double)a);
	a = 2147483646;
	trace("\na = (cfloat)%d ==> %c0x%x*2^%i ~ %g", 2147483646, _sign_(a.m), a.m, a.e, (double)a);

	 //big numbers are not implemented yet
	a = 2147483648;
	trace("\na = %d = 0x%x*2^%i ~ %g", 2147483648, _sign_(a.m),a.m, a.e, (double)a);

	a = 39887;
	trace("\na = (cfloat)%d ==> %c0x%x*2^%i ~ %g", 39887, _sign_(a.m), a.m, a.e, (double)a);
	a = 0.5625f;
	trace("\na = (cfloat)%g ==> %c0x%x*2^%i ~ %g", 0.5625f, _sign_(a.m), a.m, a.e, (double)a);
	a = 39887.5625;
	trace("\na = (cfloat)%g ==> %c0x%x*2^%i ~ %g", 39887.5625f, _sign_(a.m), a.m, a.e, (double)a);
	a = 1.5625f;
	trace("\na = (cfloat)%g ==> %c0x%x*2^%i ~ %g", 1.5625f, _sign_(a.m), a.m, a.e, (double)a);
	a = 1.5626f;
	trace("\na = (cfloat)%g ==> %c0x%x*2^%i ~ %g", 1.5626f, _sign_(a.m), a.m, a.e, (double)a);
	a = 3.2901785714285721;
	trace("\na = (cfloat)%g ==> %c0x%x*2^%i ~ %g", 3.2901785714285721, _sign_(a.m), a.m, a.e, (double)a);

	a = 1e-8f;
	trace("\na = (cfloat)%g ==> %c0x%x*2^%i ~ %g", 1e-8f, _sign_(a.m), a.m, a.e, (double)a);
	a = 1e-32f;
	trace("\na = (cfloat)%g ==> %c0x%x*2^%i ~ %g", 1e-32f, _sign_(a.m), a.m, a.e, (double)a);
	a = 1e-256f;
	trace("\na = (cfloat)%g ==> %c0x%x*2^%i ~ %g", 1e-256f, _sign_(a.m), a.m, a.e, (double)a);
	a = -1313;
	trace("\na = (cfloat)%g ==> %c0x%x*2^%i ~ %g", -1313., _sign_(a.m), a.m, a.e, (double)a);
}


void test_add()
{
	trace("\n\n *** ADDITION ***");
	double a, b, c;
	cfloat cfa, cfb, cfc;

	a = 2;
	b = 3;
	c = a + b;
	cfa = a; cfb = b; 
	cfc = cfa + cfb;
	trace("\n\ndouble: %g + %g = % g \ncfloat: %c0x%X*2^%i + %c0x%X*2^%i = %c0x%X*2^%i \n      : %g + %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 3;
	b = -2;
	c = a + b;
	cfa = a; cfb = b; 
	cfc = cfa + cfb;
	trace("\n\ndouble: %g + %g = % g \ncfloat: %c0x%X*2^%i + %c0x%X*2^%i = %c0x%X*2^%i \n      : %g + %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 2;
	b = -2;
	c = a + b;
	cfa = a; cfb = b; 
	cfc = cfa + cfb;
	trace("\n\ndouble: %g + %g = % g \ncfloat: %c0x%X*2^%i + %c0x%X*2^%i = %c0x%X*2^%i \n      : %g + %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 2;
	b = -3;
	c = a + b;
	cfa = a; cfb = b; 
	cfc = cfa + cfb;
	trace("\n\ndouble: %g + %g = % g \ncfloat: %c0x%X*2^%i + %c0x%X*2^%i = %c0x%X*2^%i \n      : %g + %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = -3;
	b = -2;
	c = a + b;
	cfa = a; cfb = b; 
	cfc = cfa + cfb;
	trace("\n\ndouble: %g + %g = % g \ncfloat: %c0x%X*2^%i + %c0x%X*2^%i = %c0x%X*2^%i \n      : %g + %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 25.5;
	b = 25.6563;
	c = a + b;
	cfa = a; cfb = b; 
	cfc = cfa + cfb;
	trace("\n\ndouble: %g + %g = % g \ncfloat: %c0x%X*2^%i + %c0x%X*2^%i = %c0x%X*2^%i \n      : %g + %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);


	a = 255;
	b = 1;
	c = a + b;
	cfa = a; cfb = b; 
	cfc = cfa + cfb;
	trace("\n\ndouble: %g + %g = % g \ncfloat: %c0x%X*2^%i + %c0x%X*2^%i = %c0x%X*2^%i \n      : %g + %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = -255;
	b = -255;
	c = a + b;
	cfa = a; cfb = b; 
	cfc = cfa + cfb;
	trace("\n\ndouble: %g + %g = % g \ncfloat: %c0x%X*2^%i + %c0x%X*2^%i = %c0x%X*2^%i \n      : %g + %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	// Here there is double round error with CustomFloat<8, 4>. One in the conversion and another in the operation.
	a = -255;
	b = -257;
	c = a + b;
	cfa = a; cfb = b; 
	cfc = cfa + cfb;
	trace("\n\ndouble: %g + %g = % g \ncfloat: %c0x%X*2^%i + %c0x%X*2^%i = %c0x%X*2^%i \n      : %g + %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = -256;
	b = -256;
	c = a + b;
	cfa = a; cfb = b; 
	cfc = cfa + cfb;
	trace("\n\ndouble: %g + %g = % g \ncfloat: %c0x%X*2^%i + %c0x%X*2^%i = %c0x%X*2^%i \n      : %g + %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	// This is the limit of the machine precission for long and CustomFloat<31, 8>.  Should work if the mantissa type is unsigned :\    
	a = 1;
	b = 2147483647;
	c = a + b;
	cfa = a; cfb = b; 
	cfc = cfa + cfb;
	trace("\n\ndouble: %g + %g = % g \ncfloat: %c0x%X*2^%i + %c0x%X*2^%i = %c0x%X*2^%i \n      : %g + %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 2147483647;
	b = 2147483647;
	c = a + b;
	cfa = a; cfb = b; 
	cfc = cfa + cfb;
	trace("\n\ndouble: %g + %g = % g \ncfloat: %c0x%X*2^%i + %c0x%X*2^%i = %c0x%X*2^%i \n      : %g + %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 2147483647;
	b = -2147483646;
	c = a + b;
	cfa = a; cfb = b; 
	cfc = cfa + cfb;
	trace("\n\ndouble: %g + %g = % g \ncfloat: %c0x%X*2^%i + %c0x%X*2^%i = %c0x%X*2^%i \n      : %g + %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 3.1416;
	b = 0;
	c = a + b;
	cfa = a; cfb = b; 
	cfc = cfa + cfb;
	trace("\n\ndouble: %g + %g = % g \ncfloat: %c0x%X*2^%i + %c0x%X*2^%i = %c0x%X*2^%i \n      : %g + %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 3.1416;
	b = 0.0001;
	c = a + b;
	cfa = a; cfb = b; 
	cfc = cfa + cfb;
	trace("\n\ndouble: %g + %g = % g \ncfloat: %c0x%X*2^%i + %c0x%X*2^%i = %c0x%X*2^%i \n      : %g + %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);
}

void test_sub()
{
	trace("\n\n*** SUBSTRACT ***");
	double a, b, c;
	cfloat cfa, cfb, cfc;

	a = 2;
	b = 3;
	c = a - b;
	cfa = a; cfb = b; 
	cfc = cfa - cfb;
	trace("\n\ndouble:%g - %g = % g \ncfloat: %c0x%X*2^%i - %c0x%X*2^%i = %c0x%X*2^%i \n      : %g - %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 0;
	b = 3;
	c = a - b;
	cfa = a; cfb = b; 
	cfc = cfa - cfb;
	trace("\n\ndouble:%g - %g = % g \ncfloat: %c0x%X*2^%i - %c0x%X*2^%i = %c0x%X*2^%i \n      : %g - %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 0;
	b = 0.0003;
	c = a - b;
	cfa = a; cfb = b; 
	cfc = cfa - cfb;
	trace("\n\ndouble:%g - %g = % g \ncfloat: %c0x%X*2^%i - %c0x%X*2^%i = %c0x%X*2^%i \n      : %g - %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 0.1;
	b = 0;
	c = a - b;
	cfa = a; cfb = b; 
	cfc = cfa - cfb;
	trace("\n\ndouble: %g - %g = % g \ncfloat: %c0x%X*2^%i - %c0x%X*2^%i = %c0x%X*2^%i \n      : %g - %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

}

void test_change_sign()
{
	trace("\n\n*** CHANGE SIGN ***");
	cfloat a = 2;
	cfloat b = -3;

	trace("\n%X = %g   ->  %X = %g ", a.m, (double)a, (-a).m, (double)(-a));
	trace("\n%X = %g   ->  %X = %g ", b.m, (double)b, (-b).m, (double)(-b));
}

void test_mul()
{
	trace("\n\n*** MULTIPLICATION ***");
	double a, b, c;
	cfloat cfa, cfb, cfc;

	a = 1;
	b = 1;
	c = a * b;
	cfa = a; cfb = b; 
	cfc = cfa * cfb;
	trace("\n\ndouble: %g * %g = % g \ncfloat: %c0x%X*2^%i * %c0x%X*2^%i = %c0x%X*2^%i \n      : %g * %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 0xa;
	b = 0x0a;
	c = a * b;
	cfa = a; cfb = b; 
	cfc = cfa * cfb;
	trace("\n\ndouble: %g * %g = % g \ncfloat: %c0x%X*2^%i * %c0x%X*2^%i = %c0x%X*2^%i \n      : %g * %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 2.4;
	b = 1.5;
	c = a * b;
	cfa = a; cfb = b; 
	cfc = cfa * cfb;
	trace("\n\ndouble: %g * %g = % g \ncfloat: %c0x%X*2^%i * %c0x%X*2^%i = %c0x%X*2^%i \n      : %g * %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 2.4;
	b = -1.5;
	c = a * b;
	cfa = a; cfb = b; 
	cfc = cfa * cfb;
	trace("\n\ndouble: %g * %g = % g \ncfloat: %c0x%X*2^%i * %c0x%X*2^%i = %c0x%X*2^%i \n      : %g * %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 0xFF;
	b = 0xFF;
	c = a * b;
	cfa = a; cfb = b; 
	cfc = cfa * cfb;
	trace("\n\ndouble: %g * %g = % g \ncfloat: %c0x%X*2^%i * %c0x%X*2^%i = %c0x%X*2^%i \n      : %g * %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 0x0;
	b = 0x0;
	c = a * b;
	cfa = a; cfb = b; 
	cfc = cfa * cfb;
	trace("\n\ndouble: %g * %g = % g \ncfloat: %c0x%X*2^%i * %c0x%X*2^%i = %c0x%X*2^%i \n      : %g * %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 1e-50;
	b = 1e-50;
	c = a * b;
	cfa = a; cfb = b; 
	cfc = cfa * cfb;
	trace("\n\ndouble: %g * %g = % g \ncfloat: %c0x%X*2^%i * %c0x%X*2^%i = %c0x%X*2^%i \n      : %g * %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 0xAA;
	b = 0x0;
	c = a * b;
	cfa = a; cfb = b; 
	cfc = cfa * cfb;
	trace("\n\ndouble: %g * %g = % g \ncfloat: %c0x%X*2^%i * %c0x%X*2^%i = %c0x%X*2^%i \n      : %g * %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 2147483647;
	b = 2147483647;
	c = a * b;
	cfa = a; cfb = b; 
	cfc = cfa * cfb;
	trace("\n\ndouble: %g * %g = % g \ncfloat: %c0x%X*2^%i * %c0x%X*2^%i = %c0x%X*2^%i \n      : %g * %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);
}

void test_div()
{
	trace("\n\n*** DIVISION ***");
	double a, b, c;
	cfloat cfa, cfb, cfc;

	a = 1;
	b = 1;
	c = a / b;
	cfa = a; cfb = b; 
	cfc = cfa / cfb;
	trace("\n\ndouble: %g / %g = % g \ncfloat: %c0x%X*2^%i / %c0x%X*2^%i = %c0x%X*2^%i \n      : %g / %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 1;
	b = 2;
	c = a / b;
	cfa = a; cfb = b; 
	cfc = cfa / cfb;
	trace("\n\ndouble: %g / %g = % g \ncfloat: %c0x%X*2^%i / %c0x%X*2^%i = %c0x%X*2^%i \n      : %g / %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 9;
	b = 2;
	c = a / b;
	cfa = a; cfb = b; 
	cfc = cfa / cfb;
	trace("\n\ndouble: %g / %g = % g \ncfloat: %c0x%X*2^%i / %c0x%X*2^%i = %c0x%X*2^%i \n      : %g / %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 1000;
	b = 3;
	c = a / b;
	cfa = a; cfb = b; 
	cfc = cfa / cfb;
	trace("\n\ndouble: %g / %g = % g \ncfloat: %c0x%X*2^%i / %c0x%X*2^%i = %c0x%X*2^%i \n      : %g / %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 0.1;
	b = -3;
	c = a / b;
	cfa = a; cfb = b; 
	cfc = cfa / cfb;
	trace("\n\ndouble: %g / %g = % g \ncfloat: %c0x%X*2^%i / %c0x%X*2^%i = %c0x%X*2^%i \n      : %g / %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 0;
	b = -3;
	c = a / b;
	cfa = a; cfb = b; 
	cfc = cfa / cfb;
	trace("\n\ndouble: %g / %g = % g \ncfloat: %c0x%X*2^%i / %c0x%X*2^%i = %c0x%X*2^%i \n      : %g / %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = -2147483647;
	b = 0.1;
	c = a / b;
	cfa = a; cfb = b; 
	cfc = cfa / cfb;
	trace("\n\ndouble: %g / %g = % g \ncfloat: %c0x%X*2^%i / %c0x%X*2^%i = %c0x%X*2^%i \n      : %g / %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);

	a = 2147483647;
	b = 0.00000001;
	c = a / b;
	cfa = a; cfb = b; 
	cfc = cfa / cfb;
	trace("\n\ndouble: %g / %g = % g \ncfloat: %c0x%X*2^%i / %c0x%X*2^%i = %c0x%X*2^%i \n      : %g / %g = %g", a, b, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfb, (double)cfc
	);
}


void test_math()
{
	trace("\n\n*** MATH ***");
	double a, c;
	cfloat cfa, cfc;

	a = 1;
	c = sin(a);
	cfa = a; 
	cfc = sin(cfa);
	trace("\nsin(%g) = %g   sin(%c0x%X*2^%i) = %c0x%X*2^%i   sin(%g) = %g", a, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfc
	);

   const double cfPI = 3.14169265335897932384626433832795;

	a = cfPI/2;
	c = sin(a);
	cfa = a; 
	cfc = sin(cfa);
	trace("\nsin(%g)=%g   sin(%c0x%X*2^%i)=%c0x%X*2^%i   sin(%g)=%g", a, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfc
	);

	a = cfPI/2;
	c = cos(a);
	cfa = a; 
	cfc = cos(cfa);
	trace("\ncos(%g)=%g   cos(%c0x%X*2^%i)=%c0x%X*2^%i   cos(%g)=%g", a, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfc
	);

	a = 5;
	c = sqrt(a);
	cfa = a; 
	cfc = sqrt(cfa);
	trace("\nsqrt(%g)=%g   sqrt(%c0x%X*2^%i)=%c0x%X*2^%i   sqrt(%g)=%g", a, c
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , (double)cfa, (double)cfc
	);
}

void test_comparison()
{
	trace("\n\n*** COMPARISON GREATER THAN***");
	double a, b;
	cfloat cfa, cfb;
	bool c;

	a = -1;
	b = -1;
	cfa = a; 
	cfb = b;

	c = (a>b);
	trace("\n%g > %g   %c0x%X*2^%i > %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");


	a = 1;
	b = 1;
	cfa = a; 
	cfb = b;

	c = (a>b);
	trace("\n%g > %g   %c0x%X*2^%i > %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");

	a = -1;
	b = 1;
	cfa = a; 
	cfb = b;

	c = (a>b);
	trace("\n%g > %g   %c0x%X*2^%i > %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");

	a = 1;
	b = -1;
	cfa = a; 
	cfb = b;

	c = (a>b);
	trace("\n%g > %g   %c0x%X*2^%i > %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");

	
	a = 0.1;
	b = 1;
	cfa = a; 
	cfb = b;

	c = (a>b);
	trace("\n%g > %g   %c0x%X*2^%i > %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");


	a = -0.1;
	b = -1;
	cfa = a; 
	cfb = b;

	c = (a>b);
	trace("\n%g > %g   %c0x%X*2^%i > %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");

	a = 1;
	b = 0.1;
	cfa = a; 
	cfb = b;

	c = (a>b);
	trace("\n%g > %g   %c0x%X*2^%i > %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");


	a = -1;
	b = -0.1;
	cfa = a; 
	cfb = b;

	c = (a>b);
	trace("\n%g > %g   %c0x%X*2^%i > %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");

	a = -1;
	b = 0;
	cfa = a; 
	cfb = b;

	c = (a>b);
	trace("\n%g > %g   %c0x%X*2^%i > %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");

	a = 0;
	b = -1;
	cfa = a; 
	cfb = b;

	c = (a>b);
	trace("\n%g > %g   %c0x%X*2^%i > %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");

	trace("\n\n*** COMPARISON LOWER THAN***");

	a = -1;
	b = -1;
	cfa = a; 
	cfb = b;

	c = (a<b);
	trace("\n%g < %g   %c0x%X*2^%i < %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");


	a = 1;
	b = 1;
	cfa = a; 
	cfb = b;

	c = (a<b);
	trace("\n%g < %g   %c0x%X*2^%i < %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");

	a = -1;
	b = 1;
	cfa = a; 
	cfb = b;

	c = (a<b);
	trace("\n%g < %g   %c0x%X*2^%i < %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");

	a = 1;
	b = -1;
	cfa = a; 
	cfb = b;

	c = (a<b);
	trace("\n%g < %g   %c0x%X*2^%i < %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");

	
	a = 0.1;
	b = 1;
	cfa = a; 
	cfb = b;

	c = (a<b);
	trace("\n%g < %g   %c0x%X*2^%i < %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");


	a = -0.1;
	b = -1;
	cfa = a; 
	cfb = b;

	c = (a<b);
	trace("\n%g < %g   %c0x%X*2^%i < %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");

	a = 1;
	b = 0.1;
	cfa = a; 
	cfb = b;

	c = (a<b);
	trace("\n%g < %g   %c0x%X*2^%i < %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");


	a = -1;
	b = -0.1;
	cfa = a; 
	cfb = b;

	c = (a<b);
	trace("\n%g < %g   %c0x%X*2^%i < %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");

	a = -1;
	b = 0;
	cfa = a; 
	cfb = b;

	c = (a<b);
	trace("\n%g < %g   %c0x%X*2^%i < %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");

	a = -0.1;
	b = -0;
	cfa = a; 
	cfb = b;

	c = (a<b);
	trace("\n%g < %g   %c0x%X*2^%i < %c0x%X*2^%i  = %s", a, b
		, _sign_(cfa.m), cfa.m, cfa.e
		, _sign_(cfb.m), cfb.m, cfb.e
		, (c == true) ? "true" : "false");

}

void test_unary_op()
{
	cfloat cfa = 0.1;
	cfloat* p_cfa = &cfa;

	trace("\n\n*** UNARY OPERATORS***");

	trace("\n%X    %c0x%X*2^%i", &cfa, _sign_(cfa.m), cfa.m, cfa.e);
	trace("\n%X    %c0x%X*2^%i", p_cfa, _sign_((*p_cfa).m), (*p_cfa).m, (*p_cfa).e);
	trace("\n%X    %c0x%X*2^%i", p_cfa, _sign_(p_cfa->m), p_cfa->m, p_cfa->e);
}

void other()
{
	float a, b, c, d;
	cfloat cfa, cfb, cfc, cfd;

	a = 25;
	b = 5.0000001f;
	c = 5;
	d = a * b - a * c;

	cfa = a; cfb = b; cfc = c; 
	cfd = cfa * cfb - cfa * cfc;
	trace("\n\ndouble: %g * %g - % g * %g = % g"
		, a, b, a, c, d);
	trace("\ncfloat: %c0x%X*2^%i * %c0x%X*2^%i - %c0x%X*2^%i * %c0x%X*2^%i = %c0x%X*2^%i"
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , _sign_(cfd.m), cfd.m, cfd.e
	);
	trace("\n  :%g * %g - % g * %g = %g"
		, (double)cfa, (double)cfb, (double)cfa, (double)cfc, (double)cfd
	);

	d = a * (b - c);
	cfa = a; cfb = b; cfc = c; 
	cfd = cfa * (cfb - cfc);
	trace("\n\ndouble: %g * (%g - %g) = % g"
		, a, b, c, d);
	trace("\ncfloat: %c0x%X*2^%i * (%c0x%X*2^%i - %c0x%X*2^%i) = %c0x%X*2^%i"
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , _sign_(cfd.m), cfd.m, cfd.e
	);
	trace("\n  :%g * (%g - %g) = %g"
		, (double)cfa, (double)cfb, (double)cfc, (double)cfd
	);

	a = 1.1001f;
	b = 1.1f;
	c = 7;
	d = a / c - b / c;

	cfa = a; cfb = b; cfc = c; 
	cfd = cfa / cfc - cfb / cfc;
	trace("\n\ndouble: %g / %g - % g / %g = % g"
		, a, b, a, c, d);
	trace("\ncfloat: %c0x%X*2^%i / %c0x%X*2^%i - %c0x%X*2^%i / %c0x%X*2^%i = %c0x%X*2^%i"
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , _sign_(cfd.m), cfd.m, cfd.e
	);
	trace("\n  :%g / %g - % g / %g = %g"
		, (double)cfa, (double)cfb, (double)cfa, (double)cfc, (double)cfd
	);

	d = (a - b) / c;
	cfa = a; cfb = b; cfc = c; 
	cfd = (cfa - cfb) / cfc;
	trace("\n\ndouble: (%g - %g) / %g = % g"
		, a, b, c, d);
	trace("\ncfloat: (%c0x%X*2^%i - %c0x%X*2^%i) / %c0x%X*2^%i = %c0x%X*2^%i"
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , _sign_(cfd.m), cfd.m, cfd.e
	);
	trace("\n  :(%g - %g) / %g = %g"
		, (double)cfa, (double)cfb, (double)cfc, (double)cfd
	);

	a = 0.11f;
	b = 0.5f;
	c = 2;
	d = a * b;
	cfa = a;
	cfb = b;
	cfc = c;
	cfd = cfa * cfb;
	trace("\n\ndouble: %g * %g = % g"
		, a, b, d);
	trace("\ncfloat: %c0x%X*2^%i * %c0x%X*2^%i = %c0x%X*2^%i"
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , _sign_(cfd.m), cfd.m, cfd.e
	);
	trace("\n  :%g * %g = %g"
		, (double)cfa, (double)cfb, (double)cfd
	);

	d = a / c;
	cfd = cfa / cfc;
	trace("\n\ndouble: %g / %g = % g"
		, a, c, d);
	trace("\ncfloat: %c0x%X*2^%i / %c0x%X*2^%i = %c0x%X*2^%i"
	   , _sign_(cfa.m), cfa.m, cfa.e
	   , _sign_(cfc.m), cfc.m, cfc.e
	   , _sign_(cfd.m), cfd.m, cfd.e
	);
	trace("\n  :%g / %g = %g"
		, (double)cfa, (double)cfc, (double)cfd
	);
}

void moretest()
{
	float a, b, c;
	cfloat cfa, cfb, cfc, cfd;

	b = 2;
	c = 1 / b;

	cfb = b; 
	cfc = 1 / cfb;
	trace("\n\ndouble: 1 / %g = % g"
		, b, c);
	trace("\ncfloat: 1 / %c0x%X*2^%i (%g) = %c0x%X*2^%i (%g)"
	   , _sign_(cfb.m), cfb.m, cfb.e
	   , (double)cfb
	   , _sign_(cfa.m), cfc.m, cfc.e
	   , (double)cfc
	);

	a = 0.000504846f;
	cfa = a;
	trace("\n\ndouble: %a > 0 = %c", a, (a>0==true) ? 'T' : 'F');
	trace("\ncfloat: %c0x%X*2^%i (%g) > 0 = %c"
		, _sign_(cfb.m), cfb.m, cfb.e
		, (double)cfa
		, (cfGreaterThan<24,8>(cfa, 0) == true) ? 'T' : 'F');}

int main(int argc, char** argv) {
	test_swap();
	test_conversion();
	test_add();
	test_sub();
	test_change_sign();
	test_mul();
	test_div();
	test_math();
	test_comparison();
	test_unary_op();

	//other();
	//moretest();

	trace("\n\npress any key");
	getchar();
    return (EXIT_SUCCESS);
}

